/*
 * Copyright (c) 1992-2002 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.8 $";
#endif

/*
 * Linux RAID specific functions
 */

#include "defs.h"
#include "strutil.h"
#include "linux-declare.h"
#include <linux/hdreg.h>
#include <linux/fs.h>

#define MAX_ARRAY_CTLRS		32		/* Max # of controllers */

/*
 * RAID Array definetion
 */
typedef struct {
    char		       *Desc;		/* Description  */
    char		       *DriverDir;	/* /proc/driver directory */
    char		       *DevDir;		/* /dev device directory */
    int				ClassType;	/* Class Type for probes */
    int			      (*Probe)();	/* Probe device func */
} Raid_t;

static int			ProbeCciss();
static int			ProbeIda();

/*
 * Known RAID Arrays
 */
static Raid_t RaidArrays[] = {
    { "Compaq Smart Array",
      "/proc/driver/cciss",	"/dev/cciss",	CT_SCSI,	ProbeCciss },
    { "Compaq Smart2 Array",
      "/proc/driver/cpqarray",	"/dev/ida",	CT_SCSI,	ProbeIda },
    0,
};

#if 0	/* This doesn't tell us anything useful right now */
/*
 * Set DevInfo device config data from DevConfig which came from /proc.
 */
static void RaidDevConfigProc(DevInfo, DevConfig)
     DevInfo_t		       *DevInfo;
     char		       *DevConfig;
{
    DiskDriveData_t	       *DriveData;
    int				Argc;
    char		      **Argv;
    char		       *cp;
    register int		i;
    Large_t			BlockSize = 0;
    Large_t			Blocks = 0;
    Large_t			Capacity = 0;	/* in MB */

    SImsg(SIM_DBG, "%s: RaidDevConfig parsing <%s>",
	  DevInfo->Name, DevConfig);

    /*
     * DevConfig looks like:
     *	blksz=512 nr_blks=35551845
     *	blksz=512 nr_blocks=35551845
     */
    Argc = StrToArgv(DevConfig, " ", &Argv, NULL, 0);
    for (i = 0; i < Argc; ++i) {
	if (EQN(Argv[i], "blksz=", 6)) 
	    BlockSize = atoL(Argv[i]+6);
	else if (EQN(Argv[i], "nr_blks=", 8)) 
	    Blocks = atoL(Argv[i]+8);
	else if (EQN(Argv[i], "nr_blocks=", 10)) 
	    Blocks = atoL(Argv[i]+10);
    }

    if (Blocks && BlockSize) {
	Capacity = Blocks / BlockSize;
    }

#if	HAVE_PRINTF_LL
    SImsg(SIM_DBG, "%s: RaidDevConfig Blocks=%lld BlockSize=%lld Cap=%lld",
	  DevInfo->Name, Blocks, BlockSize, Capacity);
#else
    SImsg(SIM_DBG, "%s: RaidDevConfig Blocks=%ld BlockSize=%ld Cap=%ld",
	  DevInfo->Name, (long)Blocks, (long)BlockSize, (long)Capacity);
#endif	/* HAVE_PRINTF_LL */

    switch (DevInfo->Type) {
    case DT_DISKDRIVE:
	if (DriveData = (DiskDriveData_t *) DevInfo->DevSpec) {
	    if (DriveData->HWdata && !DriveData->HWdata->Size)
		DriveData->HWdata->Size = (float) Capacity;
	}
	break;
    }

    if (!DevInfo->Capacity && (cp = GetSizeStr(Capacity, MBYTES)))
	DevInfo->Capacity = strdup(cp);
}
#endif	/* 0 */

/*
 * Set DevInfo device config data by various methods.
 */
static void RaidDevConfig(DevInfo, DevFile, DevConfig)
     DevInfo_t		       *DevInfo;
     char		       *DevFile;
     char		       *DevConfig;
{
    DiskDriveData_t	       *DriveData = NULL;
    DiskDrive_t		       *Disk = NULL;
    int				fd;
    int				SectSize;
    struct hd_geometry		Geometry;
    Large_t			Sectors = 0;
    char		       *cp;

    SImsg(SIM_DBG, "%s: RaidDevConfig DevFile=<%s> DevConfig=<%s>",
	  DevInfo->Name, DevFile, PRTS(DevConfig));

    switch (DevInfo->Type) {
    case DT_DISKDRIVE:
	fd = open(DevFile, O_RDWR|O_NDELAY);
	if (fd >= 0) {
	    /* Check GEOMETRY */
	    if (ioctl(fd, HDIO_GETGEO, &Geometry) == 0) {
		SImsg(SIM_DBG,
		      "%s:               GEOMETRY Heads=%d Sect=%d Cyl=%d",
		      DevFile,
		      Geometry.heads, Geometry.sectors, Geometry.cylinders);
		if (!Disk)
		    Disk = NewDiskDrive(NULL);
		Disk->Tracks = Geometry.heads;
		Disk->PhyCyl = Geometry.cylinders;
		Disk->Sect = Geometry.sectors;
	    } else {
		SImsg(SIM_DBG, "%s: ioctl HDIO_GETGEO failed: %s",
		      DevFile, SYSERR);
	    }

	    /* Query for Sector Size */
	    if (ioctl(fd, BLKSSZGET, &SectSize) == 0) {
		SImsg(SIM_DBG,
		      "%s:               BLKSSZGET Sector Size=%d",
		      DevFile, SectSize);
		if (!Disk)
		    Disk = NewDiskDrive(NULL);
		Disk->SecSize = SectSize;
	    } else {
		SImsg(SIM_DBG, "%s: ioctl BLKGETSIZE failed: %s",
		      DevFile, SYSERR);
	    }
	    if (!Disk->SecSize)
		Disk->SecSize = 512;	/* XXX Default */

	    /* Query for SIZE */
	    if (ioctl(fd, BLKGETSIZE, &Sectors) == 0) {
		SImsg(SIM_DBG,
		      "%s:               BLKGETSIZE Sectors=%ld",
		      DevFile, Sectors);
		if (!Disk)
		    Disk = NewDiskDrive(NULL);
		Disk->Size = nsect_to_mbytes(Sectors, Disk->SecSize);
	    } else {
		SImsg(SIM_DBG, "%s: ioctl BLKGETSIZE failed: %s",
		      DevFile, SYSERR);
	    }

	    (void) close(fd);
	} else {
	    SImsg(SIM_DBG, "%s: Open failed: %s", DevFile, SYSERR);
	}

	DriveData = (DiskDriveData_t *) DevInfo->DevSpec;
	if (!DriveData) {
	    DriveData = NewDiskDriveData(NULL);
	    DevInfo->DevSpec = DriveData;
	}
	DriveData->OSdata = Disk;

	if (Disk->Size && !DevInfo->Capacity)
	    if (cp = GetSizeStr((Large_t)Disk->Size, MBYTES))
		DevInfo->Capacity = strdup(cp);

	break;
    }
}

/*
 * Probe a RAID disk drive.
 * Mainly we need to build up enough info to have the real ProbeDiskDrive
 * do the heavy lifting.
 */
static int RaidProbeDisk(CtlrDevInfo, Array, DiskName, DevConfig)
     DevInfo_t		       *CtlrDevInfo;
     Raid_t		       *Array;
     char		       *DiskName;
     char		       *DevConfig;
{
    static DevData_t		DevData;
    static char			DevFile[MAXPATHLEN];
    DevInfo_t		       *DevInfo;
    char		       *cp;

    DevDataInit(&DevData);
    snprintf(DevFile, sizeof(DevFile), "%s/%s", Array->DevDir, DiskName);
    DevData.DevFile = strdup(DevFile);
    /* 
     * XXX Assume disk names look like "cNdU"
     * Extract the unit # by looking at DiskName
     */
    if (cp = strchr(DiskName, 'd')) {
	DevData.DevUnit = atoi(cp+1);
	/* Zap for DevData.DevName */
	*(cp+1) = CNULL;
    }
    DevData.DevName = DiskName;
    DevData.DevType = DT_DISKDRIVE;
    DevData.CtlrDevInfo = CtlrDevInfo;
    DevData.Flags |= DD_IS_ALIVE;

    if (DevInfo = ProbeDevice(&DevData, &CtlrDevInfo, NULL, NULL)) {
	RaidDevConfig(DevInfo, DevFile, DevConfig);
	if (AddDevice(DevInfo, &CtlrDevInfo, NULL) == 0)
	    return 1;
    }

    return 0;
}

/*
 * Probe Compaq Smart Array - cciss
 * Our preferred method of probe is to use some kind of direct C API usually
 * via ioctl's, but the cciss driver doesn't provide a means to match the
 * disk devices attached to /dev devices.  Hence, we use the cciss specifc
 * /proc text file which is prone to change without notice.
 *
 * The /proc/driver/cciss/ccissN (where N is the ctlr #) looks like:
cciss0:  Compaq Smart Array 5i Controller
       Board ID: 0x40800e11
       Firmware Version: 1.16
       Memory Address: 0xf884c000
       IRQ: 3
       Logical drives: 1
       Highest Logical Volume ID: 0
       Current Q depth: 0
       Current # commands on controller 0
       Max Q depth since init: 128
       Max # commands on controller since init: 128
       Max SG entries since init: 31
       Sequential access devices: 0

cciss/c0d0: blksz=512 nr_blocks=35553120
nr_allocs = 37869
nr_frees = 37869
 *
 * Returns the number of devices added or 0 if none.
 */
static int ProbeCciss(CtlrDevInfo, Array)
     DevInfo_t		       *CtlrDevInfo;
     Raid_t		       *Array;
{
    DevInfo_t		       *DevInfo = NULL;
    char		       *CtlrBase = "cciss";
    static int			CtlrNum;
    static char		        DevFile[MAXPATHLEN];
    static char		        DevName[MAXPATHLEN];
    size_t			DevNameLen;
    FILE		       *fp;
    static char			Buff[1024];
    char		       *Key;
    char		       *Val;
    char		       *DiskName = NULL;
    static char			DevStart[64];
    char		       *Config = NULL;
    char		       *cp;
    int				DoDesc;
    int				DevCount = 0;

    if (!CtlrDevInfo || !Array || !DevName)
	return -1;

    /*
     * XXX Assume that we are called from "lowest" PCI device to "highest"
     * so that CtlrNum will increment appropriately.  There doesn't seem to
     * be any means of matching CtlrDevInfo to the appropriate file in
     * /proc/driver/cciss/ccissN.
     */
    snprintf(DevName, sizeof(DevName), "%s%d", CtlrBase, CtlrNum++);
    snprintf(DevFile, sizeof(DevFile), "%s/%s",
	     Array->DriverDir, DevName);
    CtlrDevInfo->AltName = CtlrDevInfo->Name;
    CtlrDevInfo->Name = strdup(DevName);
    snprintf(DevStart, sizeof(DevStart), "%s/", CtlrBase);

    SImsg(SIM_DBG,
	  "ProbeCciss: DevName=<%s> DevFile=<%s>",
	  DevName, DevFile);

    fp = fopen(DevFile, "r");
    if (!fp) {
	SImsg(SIM_DBG,
	      "ProbeCciss: %s: Open for read failed: %s",
	      DevFile, SYSERR);
	return -1;
    }	

    if (Debug) {
	SImsg(SIM_DBG, "ProbeCciss: START OF <%s>", DevFile);
	snprintf(Buff, sizeof(Buff), "/bin/cat %s", DevFile);
	(void) system(Buff);
	(void) fflush(stdout);
	(void) fflush(stderr);
	SImsg(SIM_DBG, "\nProbeCciss: END OF <%s>", DevFile);
    }

    DevNameLen = strlen(DevName);
    while (fgets(Buff, sizeof(Buff), fp)) {
	if (cp = strchr(Buff, '\n'))
	    *cp = CNULL;
	if (cp = strchr(Buff, '\r'))
	    *cp = CNULL;

	if (!strlen(Buff)) {
	    /* Stop looking for description info */
	    DoDesc = FALSE;
	    continue;
	}

	if (EQN(Buff, DevName, DevNameLen)) {
	    /*
	     * Array model line
	     */
	    /* Skip to start of description */
	    for (cp = Buff + DevNameLen + 1; cp && *cp && isspace(*cp); ++cp);
	    if (cp && *cp) {
		/* Skip past vendor part of 'cp' if it's the Vendor */
		if (EQN(CtlrDevInfo->Vendor, cp, 
			strlen(CtlrDevInfo->Vendor)))
		    cp += strlen(CtlrDevInfo->Vendor) + 1;
		SImsg(SIM_DBG, "\tModel=<%s>", cp);
		CtlrDevInfo->Model = strdup(cp);
	    }
	    /* Start looking for description info */
	    DoDesc = TRUE;
	} else if (DoDesc && isspace(Buff[0])) {
	    /*
	     * This is a description entry for the array
	     */
	    Key = Val = NULL;
	    /* Skip leading white space */
	    for (Key = Buff; Key && *Key && isspace(*Key); ++Key);
	    if (cp = strchr(Key, ':')) {
		*cp = CNULL;
		/* Skip leading white space */
		for (Val = cp + 1; Val && *Val && isspace(*Val); ++Val);
	    }
	    /* Add what we got */
	    if (Key && *Key && Val && *Val) {
		SImsg(SIM_DBG, "\tDESC <%s>=<%s>", Key, Val);
		AddDesc(&(CtlrDevInfo->DescList), Key, Val, 0);
	    }
	} else if (EQN(Buff, DevStart, strlen(DevStart))) {
	    /*
	     * This is a disk device line
	     */
	    /* Stop looking for description info */
	    DoDesc = FALSE;
	    /* This line contains a disk device name */
	    DiskName = strdup(Buff + strlen(DevStart));
	    /* Terminate end of name and find start of disk config */
	    if (cp = strchr(DiskName, ':')) {
		*cp = CNULL;
		Config = cp + 1;
	    }
	    SImsg(SIM_DBG, "\tDiskName=<%s>", DiskName);
	    DevCount += RaidProbeDisk(CtlrDevInfo, Array, DiskName, Config);
	}
    }

    (void) fclose(fp);

    return DevCount;
}

/*
 * Probe Compaq Smart2 Array - ida
 * Our preferred method of probe is to use some kind of direct C API usually
 * via ioctl's, but the cciss driver doesn't provide a means to match the
 * disk devices attached to /dev devices.  Hence, we use the cciss specifc
 * /proc text file which is prone to change without notice.
 *
 * The /proc/driver/ida/idaN (where N is the ctlr #) looks like:
ida0:  Compaq Integrated Array Controller
       Board ID: 0x40400e11
       Firmware Revision: 1.42
       Controller Sig: 0xadef3e26
       Memory Address: 0xf8841000
       I/O Port: 0x2000
       IRQ: 5
       Logical drives: 1
       Highest Logical ID: 0
       Physical drives: 2

       Current Q depth: 0
       Max Q depth since init: 88

Logical Drive Info:
ida/c0d0: blksz=512 nr_blks=35551845
nr_allocs = 10504
nr_frees = 10504
 *
 * Returns the number of devices added or 0 if none.
 */
static int ProbeIda(CtlrDevInfo, Array)
     DevInfo_t		       *CtlrDevInfo;
     Raid_t		       *Array;
{
    DevInfo_t		       *DevInfo = NULL;
    char		       *CtlrBase = "ida";
    static int			CtlrNum;
    static char		        DevFile[MAXPATHLEN];
    static char		        DevName[MAXPATHLEN];
    size_t			DevNameLen;
    FILE		       *fp;
    static char			Buff[1024];
    char		       *Key;
    char		       *Val;
    char		       *DiskName = NULL;
    static char			DevStart[64];
    char		       *Config = NULL;
    char		       *cp;
    int				DoDesc;
    int				DevCount = 0;

    if (!CtlrDevInfo || !Array || !DevName)
	return -1;

    /*
     * XXX Assume that we are called from "lowest" PCI device to "highest"
     * so that CtlrNum will increment appropriately.  There doesn't seem to
     * be any means of matching CtlrDevInfo to the appropriate file in
     * /proc/driver/ida/idaN.
     */
    snprintf(DevName, sizeof(DevName), "%s%d", CtlrBase, CtlrNum++);
    snprintf(DevFile, sizeof(DevFile), "%s/%s",
	     Array->DriverDir, DevName);
    CtlrDevInfo->AltName = CtlrDevInfo->Name;
    CtlrDevInfo->Name = strdup(DevName);
    snprintf(DevStart, sizeof(DevStart), "%s/", CtlrBase);

    SImsg(SIM_DBG,
	  "ProbeIda: DevName=<%s> DevFile=<%s>",
	  DevName, DevFile);

    fp = fopen(DevFile, "r");
    if (!fp) {
	SImsg(SIM_DBG,
	      "ProbeIda: %s: Open for read failed: %s",
	      DevFile, SYSERR);
	return -1;
    }	

    if (Debug) {
	SImsg(SIM_DBG, "ProbeIda: START OF <%s>", DevFile);
	snprintf(Buff, sizeof(Buff), "/bin/cat %s", DevFile);
	(void) system(Buff);
	(void) fflush(stdout);
	(void) fflush(stderr);
	SImsg(SIM_DBG, "\nProbeIda: END OF <%s>", DevFile);
    }

    DevNameLen = strlen(DevName);
    while (fgets(Buff, sizeof(Buff), fp)) {
	if (cp = strchr(Buff, '\n'))
	    *cp = CNULL;
	if (cp = strchr(Buff, '\r'))
	    *cp = CNULL;

	if (!strlen(Buff)) {
	    /* Stop looking for description info */
	    DoDesc = FALSE;
	    continue;
	}

	if (EQN(Buff, DevName, DevNameLen)) {
	    /*
	     * Array model line
	     */
	    /* Skip to start of description */
	    for (cp = Buff + DevNameLen + 1; cp && *cp && isspace(*cp); ++cp);
	    if (cp && *cp) {
		/* Skip past vendor part of 'cp' if it's the Vendor */
		if (EQN(CtlrDevInfo->Vendor, cp, 
			strlen(CtlrDevInfo->Vendor)))
		    cp += strlen(CtlrDevInfo->Vendor) + 1;
		SImsg(SIM_DBG, "\tModel=<%s>", cp);
		CtlrDevInfo->Model = strdup(cp);
	    }
	    /* Start looking for description info */
	    DoDesc = TRUE;
	} else if (DoDesc && isspace(Buff[0])) {
	    /*
	     * This is a description entry for the array
	     */
	    Key = Val = NULL;
	    /* Skip leading white space */
	    for (Key = Buff; Key && *Key && isspace(*Key); ++Key);
	    if (cp = strchr(Key, ':')) {
		*cp = CNULL;
		/* Skip leading white space */
		for (Val = cp + 1; Val && *Val && isspace(*Val); ++Val);
	    }
	    /* Add what we got */
	    if (Key && *Key && Val && *Val) {
		SImsg(SIM_DBG, "\tDESC <%s>=<%s>", Key, Val);
		AddDesc(&(CtlrDevInfo->DescList), Key, Val, 0);
	    }
	} else if (EQN(Buff, DevStart, strlen(DevStart))) {
	    /*
	     * This is a disk device line
	     */
	    /* Stop looking for description info */
	    DoDesc = FALSE;
	    /* This line contains a disk device name */
	    DiskName = strdup(Buff + strlen(DevStart));
	    /* Terminate end of name */
	    if (cp = strchr(DiskName, ':')) {
		*cp = CNULL;
		Config = cp + 1;
	    }
	    SImsg(SIM_DBG, "\tDiskName=<%s>", DiskName);
	    DevCount += RaidProbeDisk(CtlrDevInfo, Array, DiskName, Config);
	}
    }

    (void) fclose(fp);

    return DevCount;
}

/*
 * Detect devices attached to RAID Array
 * Returns the number of devices added.
 */
static int RaidDetectArray(CtlrDevInfo, Array)
     DevInfo_t		       *CtlrDevInfo;
     Raid_t		       *Array;
{
    DevInfo_t		       *DevInfo;
    register int		u;

    SImsg(SIM_DBG,
	  "RaidDetectArray: Ctlr=<%s> Check \"%s\" DriverDir=<%s> DevDir=<%s>",
	  CtlrDevInfo->Name,
	  Array->Desc, Array->DriverDir, Array->DevDir);

    /*
     * If the DriverDir isn't present, then the array isn't present.
     * This quickly tells us if we should probe this array or not.
     */
    if (!IsDir(Array->DriverDir)) {
	SImsg(SIM_DBG,
	      "RaidDetectArray: Ctlr=<%s> DriverDir=<%s> does not exist.",
	      CtlrDevInfo->Name, Array->DriverDir);
	return -1;
    }

    if (!IsDir(Array->DevDir)) {
	SImsg(SIM_DBG,
	      "RaidDetectArray: Ctlr=<%s> DevDir=<%s> does not exist.",
	      CtlrDevInfo->Name, Array->DevDir);
	return -1;
    }

    return (*Array->Probe)(CtlrDevInfo, Array);
}

/*
 * Detect devices attached to RAID Arrays.
 */
extern DevList_t *RaidDetect(CtlrDevInfo)
     DevInfo_t		       *CtlrDevInfo;
{
    Raid_t		       *Array;

    SImsg(SIM_DBG, "RaidDetect: Array=<%s> Looking for devices",
	  CtlrDevInfo->Name);

    /*
     * Iterate through each RAID Array type until one type succeeds in
     * finding devices for this controller.
     */
    for (Array = RaidArrays; Array && Array->Desc; ++Array) {
	if (RaidDetectArray(CtlrDevInfo, Array) > 0)
	    break;
    }

    return 0;
}
