/*
 * Copyright (c) 1992-2002 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.3 $";
#endif

#include <unistd.h>
#include "defs.h"
#include "getsysconf.h"

/*
 * Table of all known sysconf(2) types
 *
 * As of SysInfo 4.2 we use a compiled tabled instead of a *.cf
 * file because some OS's use "enum" for _SC_* values so it's not
 * safe to store hardcoded values for _SC_* and assume they won't
 * change.
 */
SysConf_t SysConfTable[] = {
    /*
     * System value sizes
     */
#ifdef _SC_CHILD_MAX
    {  _SC_CHILD_MAX, 		"_SC_CHILD_MAX", 	SCT_LONG,
       "Max processes allowed to any UID" },
#endif
#ifdef _SC_OPEN_MAX
    {  _SC_OPEN_MAX,		"_SC_OPEN_MAX",		SCT_LONG,
       "Max open files per process" },
#endif
#ifdef _SC_STREAM_MAX
    {  _SC_STREAM_MAX, 		"_SC_STREAM_MAX",	SCT_LONG, 
       "Max stdio streams per process" },
#endif
#ifdef _SC_MAXPID
    {  _SC_MAXPID,		"_SC_MAXPID",		SCT_LONG,
       "Maximum pid value" },
#endif
#ifdef _SC_PASS_MAX
    {  _SC_PASS_MAX,		"_SC_PASS_MAX",		SCT_LONG,
       "Max password length" },
#endif
#ifdef _SC_LOGIN_NAME_MAX
    {  _SC_LOGIN_NAME_MAX,	"_SC_LOGIN_NAME_MAX",	SCT_LONG,
       "Max login name" },
#endif
#ifdef _SC_TTY_NAME_MAX
    {  _SC_TTY_NAME_MAX,	"_SC_TTY_NAME_MAX",	SCT_LONG,
       "Max TTY name" },
#endif
#ifdef _SC_NGROUPS_MAX
    {  _SC_NGROUPS_MAX, 	"_SC_NGROUPS_MAX",	SCT_LONG,
       "Max simultaneous groups per user" },
#endif
#ifdef _SC_TZNAME_MAX
    {  _SC_TZNAME_MAX,		"_SC_TZNAME_MAX",	SCT_LONG,
       "Max bytes in TZ environment variable" },
#endif
#ifdef _SC_PAGESIZE
    {  _SC_PAGESIZE,		"_SC_PAGESIZE",		SCT_LONG,
       "System memory page size" },
#endif
#ifdef _SC_PHYS_PAGES	
    {  _SC_PHYS_PAGES,		"_SC_PHYS_PAGES",	SCT_LONG,
       "Total number of pages of physical memory" },
#endif
#ifdef _SC_AVPHYS_PAGES
    {  _SC_AVPHYS_PAGES,	"_SC_AVPHYS_PAGE",	SCT_LONG,
       "Number of pages of physical memory not currently in use" },
#endif
    /*
     * Programming related values
     */
#ifdef _SC_JOB_CONTROL
    {  _SC_JOB_CONTROL,		"_SC_JOB_CONTROL",	SCT_BOOL,
       "Job control supported" },
#endif
#ifdef _SC_SAVED_IDS
    {  _SC_SAVED_IDS,		"_SC_SAVED_IDS",	SCT_BOOL,
       "Savid ids (seteuid()) supported" },
#endif
#ifdef _SC_ARG_MAX
    {  _SC_ARG_MAX, 		"_SC_ARG_MAX", 		SCT_LONG, 
      "Max combined size of argv[] and envp[]" },
#endif
#ifdef _SC_BC_BASE_MAX
    {  _SC_BC_BASE_MAX,		"_SC_BC_BASE_MAX",	SCT_LONG,
       "Max ibase and obase allowed by bc(1)" },
#endif
#ifdef _SC_BC_DIM_MAX
    {  _SC_BC_DIM_MAX,		"_SC_BC_DIM_MAX",	SCT_LONG,
       "Max number of elements in array by bc(1)" },
#endif
#ifdef _SC_BC_SCALE_MAX
    {  _SC_BC_SCALE_MAX,	"_SC_BC_SCALE_MAX",	SCT_LONG,
       "Max scale factor allowed by bc(1)" },
#endif
#ifdef _SC_BC_STRING_MAX
    {  _SC_BC_STRING_MAX,	"_SC_BC_STRING_MAX",	SCT_LONG,
       "Max length of strings allowed by bc(1)" },
#endif
#ifdef _SC_COLL_WEIGHTS_MAX
    {  _SC_COLL_WEIGHTS_MAX,	"_SC_COLL_WEIGHTS_MAX",	SCT_LONG,
       "Max weights for LC_COLLATE in localedef(1M)" },
#endif
#ifdef _SC_EQUIV_CLASS_MAX
    {  _SC_EQUIV_CLASS_MAX, 	"_SC_EQUIV_CLASS_MAX", 	SCT_LONG,
       "Equiv Class Max" },
#endif
#ifdef _SC_EXPR_NEST_MAX
    {  _SC_EXPR_NEST_MAX,	"_SC_EXPR_NEST_MAX",	SCT_LONG,
       "Max parenthesis nesting level for expr(1)" },
#endif
#ifdef _SC_LINE_MAX
    {  _SC_LINE_MAX,		"_SC_LINE_MAX",		SCT_LONG,
       "Max bytes in an input line" },
#endif
#ifdef _SC_RE_DUP_MAX
    {  _SC_RE_DUP_MAX,		"_SC_RE_DUP_MAX",	SCT_LONG,
       "Max repeated occurrences of RegExp in regcmp(3C)" },
#endif
#ifdef _SC_IOV_MAX
    {  _SC_IOV_MAX,		"_SC_IOV_MAX",		SCT_LONG,
       "Maximum iovec structures for readv()/writev()" },
#endif
#ifdef _SC_STACK_PROT	
    {  _SC_STACK_PROT,		"_SC_STACK_PROT",	SCT_LONG,
       "Default stack protection" },
#endif
#ifdef _SC_LIBC_VERSION
    {  _SC_LIBC_VERSION,	"_SC_LIBC_VERSION",	SCT_LONG,
       "Libc version" },
#endif
    /*
     * POSIX 1
     */
#ifdef _SC_VERSION
    {  _SC_VERSION,		"_SC_VERSION",		SCT_LONG,
       "POSIX 1 version supported" },
#endif
    /*
     * POSIX 2
     */
#ifdef _SC_2_VERSION
    {  _SC_2_VERSION,		"_SC_2_VERSION",	SCT_LONG,
       "POSIX.2 version supported" },
#endif
#ifdef _SC_2_C_BIND
    {  _SC_2_C_BIND,		"_SC_2_C_BIND",		SCT_BOOL,
       "Supports POSIX.2 C Language Bindings" },
#endif
#ifdef _SC_2_C_DEV
    {  _SC_2_C_DEV,		"_SC_2_C_DEV",		SCT_BOOL,
       "Supports POSIX.2 C Language Development Utilities" },
#endif
#ifdef _SC_2_FORT_DEV
    {  _SC_2_FORT_DEV,		"_SC_2_FORT_DEV",	SCT_BOOL,
       "Supports POSIX.2 FORTRAN Development Utilities" },
#endif
#ifdef _SC_2_FORT_RUN
    {  _SC_2_FORT_RUN,		"_SC_2_FORT_RUN",	SCT_BOOL,
       "Supports POSIX.2 FORTRAN Runtime Utilities" },
#endif
#ifdef _SC_2_SW_DEV
    {  _SC_2_SW_DEV,		"_SC_2_SW_DEV",		SCT_BOOL,
       "Supports POSIX.2 Software Development Utilities" },
#endif
#ifdef _SC_2_LOCALEDEF
    {  _SC_2_LOCALEDEF,		"_SC_2_LOCALEDEF",	SCT_BOOL,
       "Supports POSIX.2 localedef utility" },
#endif
#ifdef _SC_2_CHAR_TERM
    {  _SC_2_CHAR_TERM,		"_SC_2_CHAR_TERM",	SCT_BOOL,
       "Supports POSIX 1003.2 Terminal Type operations" },
#endif
#ifdef _SC_2_UPE
    {  _SC_2_UPE,		"_SC_2_UPE",		SCT_BOOL,
       "Supports User Portability Utilities Option" },
#endif
    /*
     * X/Open
     */
#ifdef _SC_XOPEN_VERSION
    {  _SC_XOPEN_VERSION,	"_SC_XOPEN_VERSION",	SCT_LONG,
       "X/Open portability guide version" },
#endif
#ifdef _SC_XOPEN_CRYPT
    {  _SC_XOPEN_CRYPT,		"_SC_XOPEN_CRYPT",	SCT_BOOL,
       "Supports X/Open Encryption Feature Group" },
#endif
#ifdef _SC_XOPEN_ENH_I18N
    {  _SC_XOPEN_ENH_I18N,	"_SC_XOPEN_ENH_I18N",	SCT_BOOL,
       "Supports X/Open Internationalization Feature Group" },
#endif
#ifdef _SC_XOPEN_SHM
    {  _SC_XOPEN_SHM,		"_SC_XOPEN_SHM",	SCT_BOOL,
       "Supports X/Open Shared Memory Feature Group" },
#endif
#ifdef _SC_XOPEN_XCU_VERSION
    {  _SC_XOPEN_XCU_VERSION,	"_SC_XOPEN_XCU_VERSION", SCT_LONG,
       "Version of supported XCU" },
#endif
#ifdef _SC_XOPEN_UNIX	
    {  _SC_XOPEN_UNIX,		"_SC_XOPEN_UNIX",	SCT_BOOL,
       "Supports X/Open CAE 8/1994" },
#endif
    /*
     * X/Open UNIX 98
     */
#ifdef _SC_XOPEN_LEGACY
    {  _SC_XOPEN_LEGACY,	"_SC_XOPEN_LEGACY",	SCT_BOOL,
       "Supports X/Open Legacy Feature Group" },
#endif
#ifdef _SC_XOPEN_REALTIME
    {  _SC_XOPEN_REALTIME,	"_SC_XOPEN_REALTIME",	SCT_BOOL,
       "Supports X/Open POSIX Realtime Feature Group" },
#endif
#ifdef _SC_XOPEN_REALTIME_THREADS
    {  _SC_XOPEN_REALTIME_THREADS,"_SC_XOPEN_REALTIME_THREADS", SCT_BOOL,
       "Supports X/Open POSIX Realtime Threads Feature Group" },
#endif
#ifdef _SC_XBS5_ILP32_OFF32
    {  _SC_XBS5_ILP32_OFF32,	"_SC_XBS5_ILP32_OFF32",	SCT_BOOL,
       "Supports X/Open ILP32 w/32-bit offset" },
#endif
#ifdef _SC_XBS5_ILP32_OFFBIG
    {  _SC_XBS5_ILP32_OFFBIG,	"_SC_XBS5_ILP32_OFFBIG",SCT_BOOL,
       "Supports X/Open LP32 w/64-bit offset" },
#endif
#ifdef _SC_XBS5_LP64_OFF64
    {  _SC_XBS5_LP64_OFF64,	"_SC_XBS5_LP64_OFF64",	SCT_BOOL,
       "Supports X/Open LP64, 64-bit offset" },
#endif
#ifdef _SC_XBS5_LP64_OFFBIG
    {  _SC_XBS5_LP64_OFFBIG,	"_SC_XBS5_LP64_OFFBIG",	SCT_BOOL,
       "Supports X/Open LP64, 64-bit offset" },
#endif
    /*
     * Misc
     */
#ifdef _SC_AES_OS_VERSION
    {  _SC_AES_OS_VERSION,	"_SC_AES_OS_VERSION",	SCT_LONG,
       "Version number of OSF/AES OSC supported" },
#endif
#ifdef _SC_ATEXIT_MAX
    {  _SC_ATEXIT_MAX,		"_SC_ATEXIT_MAX",	SCT_LONG,
       "Max number of register functions for atexit(2)" },
#endif
#ifdef _SC_IO_TYPE
    {  _SC_IO_TYPE,		"_SC_IO_TYPE",		SCT_LONG,
       "Type of I/O drivers" },
#endif
#ifdef _SC_SECURITY_CLASS
    {  _SC_SECURITY_CLASS,	"_SC_SECURITY_CLASS",	SCT_LONG,
       "DoD security level" },
#endif
#ifdef _SC_MSEM_LOCKID
    {  _SC_MSEM_LOCKID,		"_SC_MSEM_LOCKID",	SCT_LONG,
       "Msemaphore lock unique identifier" },
#endif
#ifdef _SC_MCAS_OFFSET
    {  _SC_MCAS_OFFSET,		"_SC_MCAS_OFFSET",	SCT_LONG,
       "Offset on gateway page of mcas_util()" },
#endif
#ifdef _SC_PROC_RSRC_MGR
    {  _SC_PROC_RSRC_MGR,	"_SC_PROC_RSRC_MGR",	SCT_LONG,
       "Process Resource Manager is configured" },
#endif
#ifdef _SC_EXEC_INTERPRETER_LENGTH
    {  _SC_EXEC_INTERPRETER_LENGTH, "_SC_EXEC_INTERPRETER_LENGTH", SCT_LONG,
       "Length of script header line" },
#endif
#ifdef _SC_SLVM_MAXNODES
    {  _SC_SLVM_MAXNODES,	"_SC_SLVM_MAXNODES",	SCT_LONG,
       "Max num of nodes supported by SLVM" },
#endif
    /*
     * Real Time values
     */
#ifdef _SC_SIGRTMIN   
    {  _SC_SIGRTMIN,		"_SC_SIGRTMIN",		SCT_LONG,
       "First POSIX.4 Realtime Signal" },
#endif
#ifdef _SC_SIGRTMAX   
    {  _SC_SIGRTMAX,		"_SC_SIGRTMAX",		SCT_LONG,
       "Last  POSIX.4 Realtime Signal" },
#endif
#ifdef _SC_RTSIG_MAX  
    {  _SC_RTSIG_MAX,		"_SC_RTSIG_MAX",	SCT_LONG,
       "Number of realtime signal values available" },
#endif
#ifdef _SC_REALTIME_SIGNALS
    {  _SC_REALTIME_SIGNALS,	"_SC_REALTIME_SIGNALS",	SCT_BOOL,
       "Supports realtime signals" },
#endif
    /*
     * CPU info
     */
#ifdef _SC_CLK_TCK
    {  _SC_CLK_TCK, 		"_SC_CLK_TCK", 		SCT_LONG, 
       "Clock ticks per second" },
#endif
#ifdef _SC_CLOCKS_PER_SEC
    {  _SC_CLOCKS_PER_SEC,	"_SC_CLOCKS_PER_SEC",	SCT_LONG,
       "Number of clock ticks per second for clock(3C)" },
#endif
#ifdef _SC_CPU_VERSION
    {  _SC_CPU_VERSION,		"_SC_CPU_VERSION",	SCT_LONG,
       "Version of CPU architecture" },
#endif
#ifdef _SC_CPU_CHIP_TYPE
    {  _SC_CPU_CHIP_TYPE,	"_SC_CPU_CHIP_TYPE",	SCT_LONG,
       "Encoded CPU chip type from PDC" },
#endif
#ifdef _SC_NPROC_CONF 
    {  _SC_NPROC_CONF ,		"_SC_NPROC_CONF ",	SCT_LONG,
       "Processors configured" },
#endif
#ifdef _SC_NPROC_ONLN 
    {  _SC_NPROC_ONLN ,		"_SC_NPROC_ONLN ",	SCT_LONG,
       "Processors on-line" },
#endif
#ifdef _SC_CPU_KEYBITS1
    {  _SC_CPU_KEYBITS1,	"_SC_CPU_KEYBITS1",	SCT_LONG,
       "Hardware key bit information" },
#endif
    /*
     * Kernel 32/64 bitsize values
     */
#ifdef _SC_KERNEL_BITS
    {  _SC_KERNEL_BITS,		"_SC_KERNEL_BITS",	SCT_LONG,
       "Running kernel is 32 or 64bit" },
#endif
#ifdef _SC_HW_32_64_CAPABLE
    {  _SC_HW_32_64_CAPABLE,	"_SC_HW_32_64_CAPABLE",	  SCT_BOOL,
       "HW is capable of running 32bit and/or 64bit OS" },
#endif
#ifdef _SC_KERNEL_IS_BIGENDIAN
    {  _SC_KERNEL_IS_BIGENDIAN,	"_SC_KERNEL_IS_BIGENDIAN", SCT_BOOL,
       "Kernel is big-endian" },
#endif
#ifdef _SC_KERN_POINTERS
    {  _SC_KERN_POINTERS,	"_SC_KERN_POINTERS",	SCT_LONG,
       "32 or 64 bit kernel value" },
#endif
#ifdef _SC_INT_MIN    
    {  _SC_INT_MIN,		"_SC_INT_MIN",		SCT_LONG,
       "Minimum value an object of type int can hold" },
#endif
#ifdef _SC_INT_MAX    
    {  _SC_INT_MAX,		"_SC_INT_MAX",		SCT_LONG,
       "Maximum value an object of type int can hold" },
#endif
#ifdef _SC_LONG_MIN   
    {  _SC_LONG_MIN,		"_SC_LONG_MIN",		SCT_LONG,
       "Minimum value an object of type long can hold" },
#endif
#ifdef _SC_LONG_MAX   
    {  _SC_LONG_MAX,		"_SC_LONG_MAX",		SCT_LONG,
       "Maximum value an object of type long can hold" },
#endif
#ifdef _SC_SSIZE_MAX  
    {  _SC_SSIZE_MAX,		"_SC_SSIZE_MAX",	SCT_LONG,
       "Maximum value an object of type ssize_t can hold" },
#endif
#ifdef _SC_WORD_BIT   
    {  _SC_WORD_BIT,		"_SC_WORD_BIT",		SCT_LONG,
       "Number of bits in a word" },
#endif
#ifdef _SC_LONG_BIT   
    {  _SC_LONG_BIT,		"_SC_LONG_BIT",		SCT_LONG,
       "Number of bits in a long" },
#endif
#ifdef _SC_SIGQUEUE_MAX
    {  _SC_SIGQUEUE_MAX,	"_SC_SIGQUEUE_MAX",	SCT_LONG,
       "Max number of queued signals" },
#endif
    /*
     * Async I/O
     */
#ifdef _SC_ASYNCHRONOUS_IO
    {  _SC_ASYNCHRONOUS_IO,	"_SC_ASYNCHRONOUS_IO",	SCT_BOOL,
       "Supports asynchronous I/O" },
#endif
#ifdef _SC_AIO_MAX
    {  _SC_AIO_MAX,		"_SC_AIO_MAX",		SCT_LONG,
       "Max simultaneous asynchronous I/O requests outstanding" },
#endif
#ifdef _SC_AIO_LISTIO_MAX
    {  _SC_AIO_LISTIO_MAX,	"_SC_AIO_LISTIO_MAX",	SCT_LONG,
       "Max queued asynchronous I/O requests outstanding" },
#endif
#ifdef _SC_AIO_PRIO_DELTA_MAX
    {  _SC_AIO_PRIO_DELTA_MAX,	"_SC_AIO_PRIO_DELTA_MAX", SCT_LONG,
       "Max amount a process can decrease its async I/O priority level" },
#endif
#ifdef _SC_PRIORITIZED_IO
    {  _SC_PRIORITIZED_IO,	"_SC_PRIORITIZED_IO",	SCT_BOOL,
       "Supports prioritized I/O" },
#endif

    /*
     * Sun Hardware specific itmes
     */
#ifdef _SC_COHER_BLKSZ
    {  _SC_COHER_BLKSZ,		"_SC_COHER_BLKSZ",	SCT_LONG,
       "Coherence block size" },
#endif
#ifdef _SC_SPLIT_CACHE
    {  _SC_SPLIT_CACHE,		"_SC_SPLIT_CACHE",	SCT_BOOL,
       "Has split cache" },
#endif
#ifdef _SC_ICACHE_SZ		
    {  _SC_ICACHE_SZ,		"_SC_ICACHE_SZ",	SCT_LONG,
       "Instruction cache size (bytes)" },
#endif
#ifdef _SC_DCACHE_SZ		
    {  _SC_DCACHE_SZ,		"_SC_DCACHE_SZ",	SCT_LONG,
       "Data cache size (bytes)" },
#endif
#ifdef _SC_ICACHE_LINESZ
    {  _SC_ICACHE_LINESZ,	"_SC_ICACHE_LINESZ",	SCT_LONG,
       "Instruction cache line size" },
#endif
#ifdef _SC_DCACHE_LINESZ
    {  _SC_DCACHE_LINESZ,	"_SC_DCACHE_LINESZ",	SCT_LONG,
       "Data cache line size" },
#endif
#ifdef _SC_ICACHE_BLKSZ
    {  _SC_ICACHE_BLKSZ,	"_SC_ICACHE_BLKSZ",	SCT_LONG,
       "Block size invalidated for icache" },
#endif
#ifdef _SC_DCACHE_BLKSZ
    {  _SC_DCACHE_BLKSZ,	"_SC_DCACHE_BLKSZ",	SCT_LONG,
       "Block size for dcache" },
#endif
#ifdef _SC_DCACHE_TBLKSZ
    {  _SC_DCACHE_TBLKSZ,	"_SC_DCACHE_TBLKSZ",	SCT_LONG,
       "long|Block size for dcache prefetch" },
#endif
#ifdef _SC_ICACHE_ASSOC
    {  _SC_ICACHE_ASSOC,	"_SC_ICACHE_ASSOC",	SCT_LONG,
       "Icache associativity 1, 2, 3 etc" },
#endif
#ifdef _SC_DCACHE_ASSOC
    {  _SC_DCACHE_ASSOC,	"_SC_DCACHE_ASSOC",	SCT_LONG,
       "Dcache associativity 1, 2, 3 etc" },
#endif

    /*
     * POSIX.1c pthreads
     */
#ifdef _SC_THREAD_DESTRUCTOR_ITERATION
    {  _SC_THREAD_DESTRUCTOR_ITERATION,	"_SC_THREAD_DESTRUCTOR_ITERATION",SCT_BOOL,
       "Supports Thread Destructor Iteration" },
#endif
#ifdef _SC_THREADS
    {  _SC_THREADS,		"_SC_THREADS",		SCT_BOOL,
       "Supports POSIX Threads" },
#endif
#ifdef _SC_THREAD_ATTR_STACKADD
    {  _SC_THREAD_ATTR_STACKADD, "_SC_THREAD_ATTR_STACKADD",	SCT_BOOL,
       "Supports Thread Stack Address Attriutes" },
#endif
#ifdef _SC_THREAD_ATTR_STACKSIZ
    {  _SC_THREAD_ATTR_STACKSIZ, "_SC_THREAD_ATTR_STACKSIZ",	SCT_BOOL,
       "Supports Thread Stack Size Attribute" },
#endif
#ifdef _SC_THREAD_PRIORITY_SCHEDULIN
    {  _SC_THREAD_PRIORITY_SCHEDULIN, "_SC_THREAD_PRIORITY_SCHEDULIN",SCT_BOOL,
       "Supports Thread Priority Scheduling" },
#endif
#ifdef _SC_THREAD_PRIO_INHERIT
    {  _SC_THREAD_PRIO_INHERIT,	"_SC_THREAD_PRIO_INHERIT",	SCT_BOOL,
       "Supports Thread Priority Inherit" },
#endif
#ifdef _SC_THREAD_PRIO_PROTECT
    {  _SC_THREAD_PRIO_PROTECT,	"_SC_THREAD_PRIO_PROTECT",	SCT_BOOL,
       "Supports Thread Priority Protection" },
#endif
#ifdef _SC_THREAD_PROCESS_SHARE
    {  _SC_THREAD_PROCESS_SHARE, "_SC_THREAD_PROCESS_SHARE",	SCT_BOOL,
       "Supports Thread Process Sharing" },
#endif
#ifdef _SC_THREAD_SAFE_FUNCTION
    {  _SC_THREAD_SAFE_FUNCTION, "_SC_THREAD_SAFE_FUNCTION",	SCT_BOOL,
       "Supports Thread Safe Functions" },
#endif
#ifdef _SC_THREAD_KEYS_MAX
    {  _SC_THREAD_KEYS_MAX,	"_SC_THREAD_KEYS_MAX",	SCT_LONG,
       "Max Thread Keys" },
#endif
#ifdef _SC_THREAD_STACK_MIN
    {  _SC_THREAD_STACK_MIN,	"_SC_THREAD_STACK_MIN",	SCT_LONG,
       "Min Thread Stack" },
#endif
#ifdef _SC_THREAD_THREADS_MAX
    {  _SC_THREAD_THREADS_MAX,	"_SC_THREAD_THREADS_MAX", SCT_LONG,
       "Max Threads per process" },
#endif

    /*
     * Misc
     */
#ifdef _SC_GETGR_R_SIZE_MAX
    {  _SC_GETGR_R_SIZE_MAX,	"_SC_GETGR_R_SIZE_MAX",	SCT_LONG,
       "Max size of group entry buffer" },
#endif
#ifdef _SC_GETPW_R_SIZE_MAX
    {  _SC_GETPW_R_SIZE_MAX,	"_SC_GETPW_R_SIZE_MAX",	SCT_LONG,
       "Max size of password entry buffer" },
#endif
#ifdef _SC_DELAYTIMER_MAX
    {  _SC_DELAYTIMER_MAX,	"_SC_DELAYTIMER_MAX",	SCT_LONG,
       "Max number of timer expiration overruns" },
#endif
#ifdef _SC_MQ_OPEN_MAX
    {  _SC_MQ_OPEN_MAX,		"_SC_MQ_OPEN_MAX",	SCT_LONG,
       "Max number of open message queue descriptors per process" },
#endif
#ifdef _SC_MQ_PRIO_MAX
    {  _SC_MQ_PRIO_MAX,		"_SC_MQ_PRIO_MAX",	SCT_LONG,
       "Max number of message priorities supported" },
#endif
#ifdef _SC_SEMAPHORES	
    {  _SC_SEMAPHORES,		"_SC_SEMAPHORES",	SCT_BOOL,
       "Supports semaphores" },
#endif
#ifdef _SC_SEM_NSEMS_MAX
    {  _SC_SEM_NSEMS_MAX,	"_SC_SEM_NSEMS_MAX",	SCT_LONG,
       "Max number of semaphores per process" },
#endif
#ifdef _SC_SEM_VALUE_MAX
    {  _SC_SEM_VALUE_MAX,	"_SC_SEM_VALUE_MAX",	SCT_LONG,
       "Max value a semaphore may have" },
#endif
#ifdef _SC_TIMER_MAX		
    {  _SC_TIMER_MAX,		"_SC_TIMER_MAX",	SCT_LONG,
       "Max number of timers per process" },
#endif
#ifdef _SC_FSYNC
    {  _SC_FSYNC,		"_SC_FSYNC",		SCT_BOOL,
       "Supports fsync File Synchronization" },
#endif
#ifdef _SC_MAPPED_FILE
    {  _SC_MAPPED_FILE,		"_SC_MAPPED_FILE",	SCT_BOOL,
       "Supports memory mapped files" },
#endif
#ifdef _SC_MEMLOCK
    {  _SC_MEMLOCK,		"_SC_MEMLOCK",		SCT_BOOL,
       "Supports process memory locking" },
#endif
#ifdef _SC_MEMLOCK_RANGE
    {  _SC_MEMLOCK_RANGE,	"_SC_MEMLOCK_RANGE",	SCT_BOOL,
       "Supports range memory locking" },
#endif
#ifdef _SC_MEMORY_PROTECTION
    {  _SC_MEMORY_PROTECTION,	"_SC_MEMORY_PROTECTION", SCT_BOOL,
       "Supports memory protection" },
#endif
#ifdef _SC_MESSAGE_PASSING
    {  _SC_MESSAGE_PASSING,	"_SC_MESSAGE_PASSING",	SCT_BOOL,
       "Supports message passing" },
#endif
#ifdef _SC_PRIORITY_SCHEDULING
    {  _SC_PRIORITY_SCHEDULING,	"_SC_PRIORITY_SCHEDULING", SCT_BOOL,
       "Supports process scheduling" },
#endif
#ifdef _SC_SHARED_MEMORY_OBJECTS
    {  _SC_SHARED_MEMORY_OBJECTS, "_SC_SHARED_MEMORY_OBJECTS", SCT_BOOL,
       "Supports shared memory objects" },
#endif

    { 0 },
};
