#
# $Id: uiMeter.pm,v 1.14 2002/03/27 18:35:39 mcooper Exp $
#
# Progress Meter
#

package uiMeter;

require 5.003;
require Exporter;

use uiGlobals;
use MacProgressBar;
use uiMisc;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   MeterCreate MeterDestroy
		   MeterProgress MeterMessage
		  );
}

my $BarWidth 		= 300;		# Width of bar
my $MeterGeoNoSplash	= '400x70';	# Width needs to be > $BarWidth
my $MeterGeoWithSplash	= '400x130';	# Width needs to be > $BarWidth
my $MeterWin;
my $Meter;
my $Canvas;
my $Bar;
my $BarFr;
my $Message;
my $Container;
my $PercentText;
my $PercentMsg;
my $MessageWidget;
my $LastPercent;

#
# Update meter message
#
sub MeterMessage {
    my $Args = shift;
    my $Msg = $Args->{Message};

    if (!$Msg) {
	return;
    }

    if (!$MessageWidget && !Tk::Exists($MessageWidget)) {
	$MessageWidget = $MeterWin->Label(-textvariable=>\$Message)
	    ->pack(-side=>'top', -anchor=>'center', -fill=>'x', -pady=>10);
    }

    $Message = $Msg;

    $MessageWidget->update;
}

#
# Create a Meter Window
#
sub MeterCreate {
    my $Args = shift;
    my $Message = $Args->{Message};
    my $Splash = $Args->{Splash};
    my $Title = $Args->{Title} || 'Progress Meter';

    if (!$MeterWin) {
	$MeterWin = MainWindow->new(-title=>WinTitle());
	$MeterWin->resizable(0,0); # Disable resizing
	my $Geo = $MeterGeoNoSplash;
	if ($Splash) {
	    if (my $image = $MeterWin->Getimage($SplashLogo)) {
		$Geo = $MeterGeoWithSplash;
		my $ImageLbl = $MeterWin->Label(-image=>$image)
		    ->pack(-side=>'top', -expand=>'1', -fill=>'x');
	    }
	}
	$MeterWin->geometry($Geo);
	#$MeterWin->overrideredirect(1);	# Remove window decorations
	# Tell window manager that we are transient
	if (my $mymaster = $MainWindow->transient()) {
	    $MeterWin->transient($mymaster);
	}
	$MainWindow->Busy(-recurse=>1);	# Change cursor to "busy" type
    }

    if ($Message) {
	MeterMessage({Message=>$Message});
    }

    if ($BarFr) {
	$BarFr->destroy() if Tk::Exists($BarFr);
    }
    if ($Bar) {
	$Bar->destroy() if Tk::Exists($Bar);
    }
    $BarFr = $MeterWin->Frame
	->pack(-side=>'bottom', -anchor=>'center', -expand=>1);
    $Bar = $BarFr->MacProgressBar(-width=>$BarWidth)
	->pack(-side=>'left',
	       -anchor=>'w');

    if ($PercentText && Tk::Exists($PercentText)) {
	$PercentText->destroy();
    }
    $PercentText = $BarFr->Label(-textvariable=>\$PercentMsg)
	->pack(-side=>'left',
	       -anchor=>'e');

    # Force update
    $MeterWin->update;
    if ($Bar) {
	$Bar->update;
    }
}

#
# Update Meter Progress
#
sub MeterProgress {
    my $Args = shift;
    my $Percent = $Args->{Percent};
    my $Message = $Args->{Message};

    if (!$MeterWin) {
	return;
    }
    if (!length($Percent)) {
	return;
    }
    #
    # Optimize updates
    #
    if ($LastPercent == $Percent && !$Message) {
	return;
    }
    $LastPercent = $Percent;

    if ($Message) {
	MeterMessage({Message=>$Message});
    }

    if ($Bar) {
	$Bar->set($Percent * 100);
    }

    $PercentMsg = sprintf("%d%%", $Percent * 100);

    # Force update
    if ($MeterWin && Tk::Exists($MeterWin)) {
	$MeterWin->update;
    }
    if ($Bar && Tk::Exists($Bar)) {
	$Bar->update;
    }
}

#
# Destroy Meter
#
sub MeterDestroy {
    if ($MeterWin && Tk::Exists($MeterWin)) {
	$MeterWin->withdraw;
    }

    # Change cursor back to normal
    $MainWindow->Unbusy(-recurse=>1);

    $MeterWin = undef;
    $MessageWidget = undef;
    $Meter = undef;
    $Canvas = undef;
    $Bar = undef;
    $Container = undef;
    $LastPercent = undef;
    $PercentText = undef;
    $PercentMsg = undef;
}

1;
